package test;

import bookRecommend.*;

public class TestBookRecommend {
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (testBook()){
			System.out.println("Pass : Book");
		}
		else{
			System.out.println("Fail : Book");
		}
		if (testMember()){
			System.out.println("Pass : Member");
		}
		else{
			System.out.println("Fail : Member");
		}
		if (testRating()){
			System.out.println("Pass : Rating");
		}
		else{
			System.out.println("Fail : Rating");
		}
		if (testBookData()){
			System.out.println("Pass : Build Book Data");
		}
		else{
			System.out.println("Fail : Build Book Data");
		}
		if (testMemberData()){
			System.out.println("Pass : Build Member Data");
		}
		else{
			System.out.println("Fail : Build Member Data");
		}
		if (testGetRecommendationByAverageRating()){
			System.out.println("Pass : Recoommend Book by Avg.");
		}
		else{
			System.out.println("Fail : Recommend Book by Avg.");
		}
		if (testGetRecommendationByCollaborativeFiltering()){
			System.out.println("Pass : Recoommend Book by Collaborative Filtering");
		}
		else{
			System.out.println("Fail : Recommend Book by Collaborative Filtering");
		}
		if (testGetRandomUnratedBook()){
			System.out.println("Pass : Get Random Unrated Book");
		}
		else{
			System.out.println("Fail : Get Random Unrated Book");
		}
	}
	
	private static boolean testBook(){
		IBook b1 = new Book("author", "title", 127);
		if (!b1.getAuthor().equals("author")){
			return false;
		}
		if (!b1.getTitle().equals("title")){
			return false;
		}
		if (b1.getID()!=127){
			return false;
		}
		IBook b2 = new Book("author","title",23);
		if (!b2.toString().equals("author,title")){
			return false;
		}
		return true;
	}
	
	private static boolean testMember(){
		IMember m1 = new Member("name", 1);
		if (!m1.getName().equals("name")){
			return false;
		}
		if (m1.getID()!=1){
			return false;
		}
		IMember m2 = new Member("name",2);
		if(m1.equals(m2)){
			return false;
		}
		if (!m1.hasReadAllBooksInSystem(0)){
			return false;
		}
		if (m1.getRatingArray(0)!=null){
			return false;
		}
		return true;
	}
	
	private static boolean testRating(){
		IMember m1 = new Member("name", 1);
		IRating r1 = new Rating(0,m1, -5);
		if (r1.getBookID()!=0){
			return false;
		}
		if (!r1.getMember().equals(m1)){
			return false;
		}
		if (r1.getRating()!=-5){
			return false;
		}
		return true;
	}
	
	private static boolean testBookData(){
		IBook[] books = Book.buildBookData("smallbooks.txt");
		if (books==null){
			return false;
		}
		if (books.length!=3){
			return false;
		}
		if (!books[0].getAuthor().equals("Douglas Adams")){
			return false;
		}
		if (!books[1].getAuthor().equals("Richard Adams")){
			return false;
		}
		if (!books[2].getAuthor().equals("Mitch Albom")){
			return false;
		}
		if (!books[0].getTitle().equals("The Hitchhiker's Guide To The Galaxy")){
			return false;
		}
		if (!books[1].getTitle().equals("Watership Down")){
			return false;
		}
		if (!books[2].getTitle().equals("The Five People You Meet in Heaven")){
			return false;
		}
		if (books[0].getID()!=0){
			return false;
		}
		if (books[1].getID()!=1){
			return false;
		}
		if (books[2].getID()!=2){
			return false;
		}
		return true;
	}
	
	private static boolean testMemberData(){
		IMember[] members = Member.buildMemberData("smallratings.txt");
		if (members==null){
			return false;
		}
		if (members.length!=4){
			return false;
		}
		if (!members[3].getName().equals("Oren")){
			return false;
		}
		if (members[3].hasReadAllBooksInSystem(3)){
			return false;
		}
		int[] correctRatings = {1,3,0};
		int[] actualRatings = members[3].getRatingArray(3);
		if ((correctRatings[0]!=actualRatings[0])||(correctRatings[1]!=actualRatings[1])||(correctRatings[2]!=actualRatings[2])){
			return false;
		}
		return true;
	}
	
	private static boolean testGetRecommendationByAverageRating(){
		IBook[] books = Book.buildBookData("books.txt");
		IMember[] members = Member.buildMemberData("ratings.txt");
		if ((books==null)||(members==null)){
			return false;
		}
		IMember m1 = members[0];
		IBook result = BookRecommend.getRecommendationByAverageRating(members, books, m1);
		if (result==null){
			return false;
		}
		if (!result.getTitle().equals("Sabriel")){
			return false;
		}
		result = BookRecommend.getRecommendationByAverageRating(members, books, null);
		if (result!=null){
			return false;
		}
		return true;
	}
	
	private static boolean testGetRecommendationByCollaborativeFiltering(){
		IBook[] books = Book.buildBookData("books.txt");
		IMember[] members = Member.buildMemberData("ratings.txt");
		if ((books==null)||(members==null)){
			return false;
		}
		IMember m1 = members[0];
		IBook result = BookRecommend.getRecommendationByCollaborativeFiltering(members, books, m1);
		if (result==null){
			return false;
		}
		if (!result.getTitle().equals("Flowers For Algernon")){
			return false;
		}
		IMember m2 = members[6];
		result = BookRecommend.getRecommendationByCollaborativeFiltering(members, books, m2);
		if (result==null){
			return false;
		}
		if (!result.getTitle().equals("Ender's Game")){
			return false;
		}
		result = BookRecommend.getRecommendationByCollaborativeFiltering(members, books, null);
		if (result!=null){
			return false;
		}
		return true;
	}
	
	private static boolean testGetRandomUnratedBook(){
		IBook[] books = Book.buildBookData("books.txt");
		IMember[] members = Member.buildMemberData("ratings.txt");
		if ((books==null)||(members==null)){
			return false;
		}
		IMember m1 = members[0];
		IBook result = BookRecommend.getRandomUnratedBook(books, m1);
		if (result==null){
			return false;
		}
		if (m1.getRatingArray(55)[result.getID()]!=0){
			return false;
		}
		result = BookRecommend.getRandomUnratedBook(books, null);
		if (result!=null){
			return false;
		}
		return true;
	}
}
